Re: [PATCH 1/6] add open_nofollow() helper
- From
Jeff King <peff@peff.net>
- Date
- Feb 16, 2021, 14:54 UTC
- Message-ID
- <YCvcoPnDAEWNcv+h@coredump.intra.peff.net>
- In-Reply-To
- <YCvaRpQs99TBaCrQ@coredump.intra.peff.net>
On Tue, Feb 16, 2021 at 09:44:23AM -0500, Jeff King wrote:
Show 15 quoted lines
> - we can determine whether the path is a symlink with lstat(). > > This is slower (two syscalls instead of one), but that may be > acceptable for infrequent uses like looking up .gitattributes files > (especially because we can get away with a single syscall for the > common case of ENOENT). > > It's also racy, but should be sufficient for our needs (we are > worried about in-tree symlinks that we ourselves would have > previously created). We could make it non-racy at the cost of making > it even slower, by doing an fstat() on the opened descriptor and > comparing the dev/ino fields to the original lstat(). > > This patch implements the lstat() option in its slightly-faster racy > form.
I manually compared the performance of the O_NOFOLLOW and fallback paths by running:
git ls-files | git check-attr --stdin -a
on linux.git, which would try to look at quite a few in-tree attribute files. The slowdown didn't seem measurable (in fact, the fallback seems about 1% faster even over many trials, but I think it's just noise). Both take ~50ms.
Which makes sense, because there's only one .gitattributes file, so most of the lookups are hitting ENOENT on the lstat() and not even calling open(). If I do:
find * -type d | sed 's,$,/.gitattributes,' | xargs touch
then the O_NOFOLLOW case takes ~54ms (which makes sense; we're opening a bunch of extra empty attribute files), and the fallback case goes to ~56ms. So the difference becomes measurable, but I wonder if it's worth even caring about. That's 2ms on a pathological case. I'd even be tempted to implement the non-racy version with the extra fstat(). I don't think we need it, but just as a least-surprise thing.
This is all on Linux, of course. Perhaps other systems with slower syscalls may be more impacted.
-Peff